@@ -63,6 +63,24 @@ class SFLvaultAccess(object):
 
         self.setup_timeout = 300
 
+    def _log_any(self, log_func, msg, data):
+        # Need to do that for user-setup
+        if self.myself_username == None and self.myself_id == None:
+            head = "NO USER - "
+        else:
+            head = "User: u#d%d - %s - " % (self.myself_id, self.myself_username)
+        log_func(head + msg % data)
+
+    def log_e(self, msg, data=None):
+        self._log_any(log.error, msg, data)
+
+    def log_i(self, msg, data=None):
+        self._log_any(log.info, msg, data)
+
+    def log_w(self, msg, data=None):
+        self._log_any(log.warning, msg, data)
+
+
     def user_setup(self, username, pubkey):
         """Setup the user's account"""
         from pylons import app_globals
@@ -70,12 +88,18 @@ class SFLvaultAccess(object):
         u = query(User).filter_by(username=username).first()
 
         if u is None:
+            self.log_i('No such temporary user %(username)s, add it first',
+                       {"username": username})
             return vaultMsg(False, 'No such temporary user %s, add it first' % username)
         
         if u.setup_expired():
+            self.log_i('Setup expired for user %(username)s',
+                       {"username": username})
             return vaultMsg(False, 'Setup expired for user %s' % username)
 
         if u.pubkey:
+            self.log_i('User %(username)s already has a publics',
+                       {"username": username})
             return vaultMsg(False, 'User %s already has a public ' \
                                 'key stored' % username)
 
@@ -87,6 +111,8 @@ class SFLvaultAccess(object):
         meta.Session.add(u)
         meta.Session.commit()
 
+        self.log_i('User setup complete for %(username)s',
+                   {"username": username})
         return vaultMsg(True, 'User setup complete for %s' % username)
 
 
@@ -105,6 +131,7 @@ class SFLvaultAccess(object):
 
             meta.Session.add(usr)
             
+            self.log_i('User %(username)s added.', {"username": username})
             msg = 'added'
         elif usr.waiting_setup:
             if usr.waiting_setup < datetime.now():
@@ -112,11 +139,16 @@ class SFLvaultAccess(object):
                 usr.waiting_setup = datetime.now() + \
                                     timedelta(0, int(self.setup_timeout))
             
+                self.log_i('updated (had setup timeout expired).', {})
                 msg = 'updated (had setup timeout expired)'
             else:
+                self.log_i('User %(username)s is waiting for setup.',
+                       {"username": username})
                 return vaultMsg(False, "User %s is waiting for setup" % \
                                 username)
         else:
+            self.log_i('User %(username)s already exists.',
+                       {"username": username})
             return vaultMsg(False, 'User %s already exists.' % username)
 
         meta.Session.commit()
@@ -138,6 +170,8 @@ class SFLvaultAccess(object):
         try:
             usr = model.get_user(user)
         except LookupError, e:
+            self.log_w('User UNsuccessfully deleted: %(error)s',
+                        {'error': str(e)})
             return vaultMsg(False, str(e))
 
         t1 = model.usergroups_table
@@ -145,6 +179,8 @@ class SFLvaultAccess(object):
         username = usr.username
         meta.Session.delete(usr)
         meta.Session.commit()
+        self.log_i('User %(username)s successfully deleted.',
+                   {"username": username})
         return vaultMsg(True, "User %s successfully deleted" % username)
 
 
@@ -207,6 +243,7 @@ class SFLvaultAccess(object):
         try:
             s = query(Service).filter_by(id=service_id).one()
         except InvalidReq, e:
+            self.log_i('Service not found: %(error)s', {"error": str(e)})
             return vaultMsg(False, "Service not found: %s" % str(e))
 
         #Save:
@@ -226,9 +263,13 @@ class SFLvaultAccess(object):
             s.notes = data['notes']
         if 'url' in data:
             s.url = data['url']
+        if 'metadata' in data:
+            s.metadata = data['metadata']
 
         meta.Session.commit()
 
+        self.log_i('Service s#(service_id)s saved successfully' ,
+                   {"service_id": service_id})
         return vaultMsg(True, "Service s#%s saved successfully" % service_id)
 
     def _service_get_data(self, service_id, group_id=None, with_groups=False):
@@ -236,6 +277,8 @@ class SFLvaultAccess(object):
         try:
             s = query(Service).filter_by(id=service_id).one()
         except InvalidReq, e:
+            self.log_w('Service not found: %(service_id)s (%(error)s)',
+                       {"service_id": service_id, "error": str(e)})
             raise VaultError("Service not found: %s (%s)" % (service_id,
                                                              str(e)))
         # unused
@@ -289,7 +332,7 @@ class SFLvaultAccess(object):
                'groups_list': groups_list,
                'parent_service_id': s.parent_service_id,
                'secret_last_modified': s.secret_last_modified,
-               'metadata': s.metadata or '',
+               'metadata': s.metadata or {},
                'notes': s.notes or ''}
 
         return out
@@ -303,6 +346,7 @@ class SFLvaultAccess(object):
                 data = self._service_get_data(service_id,
                                               with_groups=with_groups)
             except VaultError, e:
+                self.log_e('Show service: %(error)s', {"error": str(e)})
                 return vaultMsg(False, str(e))
             
             out.append(data)
@@ -315,10 +359,13 @@ class SFLvaultAccess(object):
 
             # check if we're not in an infinite loop!
             if service_id in [x['id'] for x in out]:
+                self.log_e('Circular references of parent services, aborting.', {})
                 return vaultMsg(False, "Circular references of parent services, aborting.")
 
         out.reverse()
 
+
+        self.log_i('Service shown: %(service_id)s', {"service_id": service_id})
         return vaultMsg(True, "Here are the services", {'services': out})    
 
     
@@ -343,6 +390,7 @@ class SFLvaultAccess(object):
         newfilters = {}
         if filters:
             if not isinstance(filters, dict):
+                self.log_e('Search: filters must be a dictionary')
                 return vaultMsg(False, "filters must be a dictionary")
             for flt in filter_types:
                 # Skip filters that aren't specified
@@ -356,6 +404,7 @@ class SFLvaultAccess(object):
                 try:
                     newfilters[flt] = model.get_objects_ids(filters[flt], flt)
                 except ValueError, e:
+                    self.log_e('Search error: %(error)s' % {"error": str(e)})
                     return vaultMsg(False, str(e))
 
         search = model.search_query(search_query, newfilters, verbose)
@@ -399,6 +448,8 @@ class SFLvaultAccess(object):
 
 
         # Return 'out', in a nicely structured hierarchical form.
+        #self.log_i('Search successfull for: %(search)s',
+        #            {'search': search_query})
         return vaultMsg(True, "Here are the search results", {'results': out})
 
 
@@ -407,6 +458,7 @@ class SFLvaultAccess(object):
         try:
             cust = query(Customer).filter_by(id=customer_id).one()
         except InvalidReq, e:
+            self.log_i('Customer not found: %(error)s', {"error": str(e)})
             return vaultMsg(False, "Customer not found: %s" % str(e))
 
         out = {'id': cust.id,
@@ -419,6 +471,8 @@ class SFLvaultAccess(object):
         try:
             cust = query(Customer).filter_by(id=customer_id).one()
         except InvalidReq, e:
+            self.log_w('Customer not found: %(customer_id)s',
+                       {"customer_id": customer_id})
             return vaultMsg(False, "Customer not found: %s" % str(e))
 
         if 'name' in data:
@@ -426,6 +480,8 @@ class SFLvaultAccess(object):
 
         meta.Session.commit()
 
+        self.log_i('Customer c#%(customer_id)s saved successfully)s',
+                  {"customer_id": customer_id})
         return vaultMsg(True, "Customer c#%s saved successfully" % customer_id)
 
 
@@ -444,6 +500,7 @@ class SFLvaultAccess(object):
         
         meta.Session.commit()
 
+        self.log_i('Customer add: c#%(customer_id)s', {"customer_id": nc.id})
         return vaultMsg(True, 'Customer added', {'customer_id': nc.id})
 
 
@@ -452,6 +509,8 @@ class SFLvaultAccess(object):
         try:
             m = query(Machine).filter_by(id=machine_id).one()
         except InvalidReq, e:
+            self.log_w('Machine m#%(machine_id)s saved successfully',
+                   {"machine_id": machine_id})
             return vaultMsg(False, "Machine not found: %s" % str(e))
 
         if 'customer_id' in data:
@@ -462,6 +521,8 @@ class SFLvaultAccess(object):
                 m.__setattr__(x, data[x])
         meta.Session.commit()
 
+        self.log_i('Machine m#%(machine_id)s saved successfully',
+                   {"machine_id": machine_id})
         return vaultMsg(True, "Machine m#%s saved successfully" % machine_id)
 
     def machine_get(self, machine_id):
@@ -501,11 +562,12 @@ class SFLvaultAccess(object):
         
         meta.Session.commit()
 
+        self.log_i('Machine added: m#%(machine_id)s', {"machine_id": nm.id})
         return vaultMsg(True, "Machine added.", {'machine_id': nm.id})
 
 
     def service_add(self, machine_id, parent_service_id, url,
-                    group_ids, secret, notes):
+                    group_ids, secret, notes, metadata):
         # Get groups
         try:
             groups, group_ids = model.get_objects_list(group_ids, 'groups',
@@ -534,6 +596,7 @@ class SFLvaultAccess(object):
         ns.secret = ciphertext
         ns.secret_last_modified = datetime.now()
         ns.notes = notes
+        ns.metadata = metadata
 
         meta.Session.save(ns)
 
